SystemOrganization addCategory: #'Rest-Client-HTTP'!

Object subclass: #RestHttpsResponse
	instanceVariableNames: 'content headers responseCode url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestHttpsResponse class methodsFor: 'as yet unclassified' stamp: 'topa 2/19/2009 19:47'!
responseCode: aNumberCode headers: aDictionary content: aString url: aUrl

	^ self new
		responseCode: aNumberCode;
		headers: aDictionary;
		content: aString;
		url: aUrl! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:01'!
content
	^ content! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:45'!
content: anObject
	content := anObject! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:03'!
getHeader: name 
	^self getHeader: name  default: nil! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:03'!
getHeader: name  default: defaultValue
	^headers at: name  ifAbsent: [ defaultValue ]! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:45'!
headers
	^ headers! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:45'!
headers: anObject
	headers := anObject! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'SR 2/19/2009 16:03'!
responseCode
	^ responseCode! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:46'!
responseCode: anObject
	responseCode := anObject! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:46'!
url
	^ url! !

!RestHttpsResponse methodsFor: 'accessing' stamp: 'topa 2/19/2009 17:46'!
url: anObject
	url := anObject! !

Object subclass: #RestHttpsSocket
	instanceVariableNames: 'conn headers responseCode request content url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestHttpsSocket class methodsFor: 'private' stamp: 'SR 2/19/2009 16:58'!
defaultPortForSchema: aSchema
	"default port to connect on"
	(aSchema = 'http') ifTrue: [^ 80].
	(aSchema = 'https') ifTrue: [^ 443].! !

!RestHttpsSocket class methodsFor: 'private' stamp: 'SR 2/19/2009 15:22'!
defaultSocketForSchema: aSchema
	"default port to connect on"
	^ {'http' -> HTTPSocket. 'https' -> HttpsSocket} asDictionary at: aSchema! !

!RestHttpsSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
deleteDocumentFromUrl: url request: requestString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'DELETE'
		request: requestString
		content: nil! !

!RestHttpsSocket class methodsFor: 'get the page' stamp: 'topa 2/19/2009 19:57'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: aContentString 
	| serverName sock |
	Socket initializeNetwork.

	serverName := url authority.
	3 timesRepeat: [
		sock := self new 
			url: url;
			request: requestString;
			content: aContentString.
		(sock connectAndWaitUntil: (Time millisecondClockValue + 10000 truncated))
			ifTrue: [^ sock doVerb: aVerb]].
	Socket deadServer: serverName.
	sock destroy.
	^ 'Server ' , serverName , ' is not responding'! !

!RestHttpsSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 14:07'!
expandUrl: newUrl ip: byteArrayIP port: portNum

	(newUrl beginsWith: '../') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: (newUrl allButFirst: 2)]].
	(newUrl beginsWith: '/') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: newUrl]].
	^ newUrl
! !

!RestHttpsSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
getDocumentFromUrl: url request: requestString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'GET'
		request: requestString
		content: nil! !

!RestHttpsSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
postDocumentFromUrl: url request: requestString content: contentString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'POST'
		request: requestString
		content: contentString! !

!RestHttpsSocket class methodsFor: 'get the page' stamp: 'SR 2/19/2009 16:15'!
putDocumentFromUrl: url request: requestString content: contentString 
	^ self 
		doHTTPRequestFromUrl: url
		verb: 'PUT'
		request: requestString
		content: contentString! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'SR 2/19/2009 14:07'!
conn
	"Answer the value of conn"

	^ conn! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'SR 2/19/2009 14:07'!
conn: anObject
	"Set the value of conn"

	conn := anObject! !

!RestHttpsSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connect
	| serverAddr |
	serverAddr := (NetNameResolver addressForName: self serverName timeout: 20)
		ifNil: [self error: 'Could not resolve the server named: ', self serverName].
		
	self connectTo: serverAddr port: self port! !

!RestHttpsSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connectAndWaitUntil: aTimeout

	^ self connect; waitForConnectionUntil: aTimeout! !

!RestHttpsSocket methodsFor: 'api' stamp: 'cs 2/19/2009 20:33'!
connectTo: addr port: aPort

	self conn:
		(self usesSSL
			ifTrue: [SSLSocket newConnectionAddress: addr port: aPort version: SSLProtocolVersion tls10]
			ifFalse: [Socket new connectTo: addr port: aPort]
		).
! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
content
	^ content! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
content: anObject
	content := anObject! !

!RestHttpsSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
destroy

	self conn close! !

!RestHttpsSocket methodsFor: 'api' stamp: 'SR 2/20/2009 13:37'!
doVerb: aHTTPVerb
	| cmd stream |
	cmd := 	aHTTPVerb,' ', self url fullPath, ' HTTP/1.1', String crlf, 
				'Host: ', self serverName , ':', self port printString, String crlf, 
				(self request ifEmpty: [''] ifNotEmpty: [self request, String crlf]).
					
	self content ifNotNil: [
		cmd := cmd, 
					'Content-type: multipart/form-data', String crlf, "was:  application/x-www-form-urlencoded"
					'Content-length: ', self content size printString , String crlf].
	self sendCommand: cmd.
	"headers are sent."					
	self content ifNotNil: [self sendCommand: self content].
	stream := self getData readStream.
	self headers: (stream upToAll: String crlf , String crlf). "parses header, too"
	^ RestHttpsResponse 
			responseCode: self responseCode  
			headers: self headers
			content: (self getRestOfData: stream upToEnd)
			url: self url! !

!RestHttpsSocket methodsFor: 'api' stamp: 'cs 2/19/2009 21:57'!
getData

	| data |
	data := self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData].
	[self conn isConnected and: [(data asString findString: String crlfcrlf) == 0]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveData]).].
	^ data asString! !

!RestHttpsSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
getHeader: name 
	^self getHeader: name  default: nil! !

!RestHttpsSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
getHeader: name  default: defaultValue
	^headers at: name  ifAbsent: [ defaultValue ]! !

!RestHttpsSocket methodsFor: 'api' stamp: 'cs 2/19/2009 22:06'!
getRestOfData: begin

	| data length |
	length := self getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].
	data := begin, (self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveAvailableData]).
	[self conn isConnected and: [length isNil or: [data size < length]]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveAvailableData]).].
	^ data asString! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 19:52'!
headers
	^ headers! !

!RestHttpsSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:53'!
headers: headerText
	"set the headers.  Then getHeader: can be used"

	"divide into basic lines"
	| lines foldedLines i statusLine |
	lines := headerText findTokens: (String with: Character cr  with: Character linefeed).
	statusLine := lines first.
	lines := lines copyFrom: 2 to: lines size.

	"parse the status (pretty trivial right now)"
	responseCode := (statusLine findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines do: [ :line |
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	headers := Dictionary new.
	foldedLines do: [ :line |
		i := line indexOf: $:.
		i > 0 ifTrue: [
			self headers 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) withBlanksTrimmed ] ].
! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'cs 2/19/2009 20:34'!
isConnecting
	^ self usesSSL
		ifTrue: [self conn isConnecting ]
		ifFalse: [self conn isWaitingForConnection ]! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 19:18'!
port
	^ 	self url port 
		ifNil:  [self class defaultPortForSchema: self url schemeName]
		ifNotNil: [self url port]
! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
request
	^ request! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:12'!
request: anObject
	request := anObject! !

!RestHttpsSocket methodsFor: 'api' stamp: 'SR 2/19/2009 14:07'!
responseCode
	^responseCode! !

!RestHttpsSocket methodsFor: 'api' stamp: 'cs 2/19/2009 20:35'!
sendCommand: cmd
	
	self usesSSL
		ifTrue: [self conn send: cmd, String crlf.]
		ifFalse: [self conn sendData: cmd, String crlf.]! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:58'!
serverName
	^ self url authority! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:23'!
url
	^ url! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'topa 2/19/2009 18:23'!
url: anObject
	url := anObject! !

!RestHttpsSocket methodsFor: 'accessing' stamp: 'test 2/19/2009 20:42'!
usesSSL
	^ url schemeName = 'https'! !

!RestHttpsSocket methodsFor: 'api' stamp: 'cs 2/19/2009 20:33'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. Return true if it is established by the deadline, false if not."

	[(self isConnecting) and: [Time millisecondClockValue < deadline]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait].
	^ self conn isConnected
! !

!HTTPSocket class methodsFor: '*rest-client' stamp: 'cs 2/15/2009 15:47'!
httpDelete: t1
	| t5 |
	t5 := self
				httpDeleteDocument: t1
				args: nil
				accept: '*/*'
				request: ''.
	t5 isString
		ifTrue: [^ t5].
	^ (RWBinaryOrTextStream with: t5 content) reset! !

!HTTPSocket class methodsFor: '*rest-client' stamp: 'cs 2/15/2009 15:44'!
httpDeleteDocument: t1 args: t2 accept: t3 request: t4 
	| t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 t16 t17 t18 t19 t20 |
	Socket initializeNetwork.
	t11 := (t1 asLowercase beginsWith: 'http://')
				ifTrue: [t1 copyFrom: 8 to: t1 size]
				ifFalse: [t1].
	t11 := t11 copyUpTo: $#.
	t5 := t11 copyUpTo: $/.
	t12 := t11 copyFrom: t5 size + 1 to: t11 size.
	(t5 includes: $:)
		ifTrue: [t16 := t5 indexOf: $:.
			t7 := (t5 copyFrom: t16 + 1 to: t5 size) asNumber.
			t5 := t5 copyFrom: 1 to: t16 - 1]
		ifFalse: [t7 := self defaultPort].
	t12 size = 0
		ifTrue: [t12 := '/'].
	t2
		ifNotNil: [t12 := t12
						, (self argString: t2)].
	(self shouldUseProxy: t5)
		ifTrue: [t12 := 'http://' , t5 , ':' , t7 printString , t12.
			t17 := self httpProxyServer.
			t18 := self httpProxyPort]
		ifFalse: [t17 := t5.
			t18 := t7].
	t6 := NetNameResolver addressForName: t17 timeout: 20.
	t6
		ifNil: [^ 'Could not resolve the server named: ' , t17].
	3
		timesRepeat: [t8 := HTTPSocket new.
			t8 connectTo: t6 port: t18.
			(t8
					waitForConnectionUntil: (self deadlineSecs: 30))
				ifFalse: [Socket deadServer: t17.
					t8 destroy.
					^ 'Server ' , t17 , ' is not responding'].
			t8 sendCommand: 'DELETE ' , t12 , ' HTTP/1.0' , CrLf
					, (t3
							ifNil: ['']
							ifNotNil: ['ACCEPT: ' , t3 , CrLf]) , 'ACCEPT: text/html' , CrLf , HTTPBlabEmail , t4 , self userAgentString , CrLf , 'Host: ' , t5 , ':' , t7 printString , CrLf.
			t13 := t8
						getResponseUpTo: CrLf , CrLf
						ignoring: (String with: CR).
			t9 := t13 at: 1.
			t14 := t13 at: 3.
			t9 isEmpty
				ifTrue: [t15 := 'server aborted early']
				ifFalse: [t8 header: t9.
					t10 := t8 getHeader: 'content-length'.
					t10
						ifNotNil: [t10 := t10 asNumber].
					t19 := t8 getHeader: 'content-type'.
					t8 responseCode first = $3
						ifTrue: [t20 := t8 getHeader: 'location'.
							t20
								ifNotNil: [Transcript show: 'redirecting to ' , t20;
										 cr.
									t8 destroy.
									t20 := self
												expandUrl: t20
												ip: t6
												port: t18.
									^ self
										httpGetDocument: t20
										args: t2
										accept: t3
										request: t4]].
					t15 := t8 getRestOfBuffer: t14 totalLength: t10.
					t8 responseCode first = $4
						ifTrue: [^ t9 , t15 contents].
					nil].
			t8 destroy.
			t15 class ~~ String
				ifTrue: [^ MIMEDocument
						contentType: t19
						content: t15 contents
						url: t1].
			t15 = 'server aborted early'
				ifTrue: [^ t15].
			nil].
	{'HTTPSocket class>>httpGetDocument:args:accept:request:'. t15. t1} inspect.
	^ 'some other bad thing happened!!'! !

!HTTPSocket class methodsFor: '*rest-client' stamp: 'cs 2/15/2009 15:50'!
httpPut: t1 args: t2
	| t4 |
	t4 := self
				httpPutDocument: t1
				args: t2
				accept: '*/*'
				request: ''.
	t4 isString
		ifTrue: [^ t4].
	^ RWBinaryOrTextStream with: t4 content! !

!HTTPSocket class methodsFor: '*rest-client' stamp: 'cs 2/16/2009 01:42'!
httpPutDocument: t1 args: t2 accept: t3 request: t4 
	| t5 t6 t7 t8 t9 t10 t11 t12 t13 t14 t15 |
	Socket initializeNetwork.
	t14 := Url absoluteFromText: t1.
	t8 := t14 fullPath.
	t15 := t2
				ifNil: ['']
				ifNotNil: [t15 := self argString: t2.
					t15 first = $?
						ifTrue: [t15 := t15 copyFrom: 2 to: t15 size]].
	t5 := HTTPSocket new.
	t5 := self
				initHTTPSocket: t14
				wait: (self deadlineSecs: 30)
				ifError: [:t16 | ^ t16].
	t5 sendCommand: 'PUT ' , t8 , ' HTTP/1.0' , CrLf
			, (t3
					ifNil: ['']
					ifNotNil: ['ACCEPT: ' , t3 , CrLf]) , 'ACCEPT: text/html' , CrLf , HTTPProxyCredentials , HTTPBlabEmail , t4 , self userAgentString , CrLf , 'Content-type: application/x-www-form-urlencoded' , CrLf , 'Content-length: ' , t15 size printString , CrLf , 'Host: ' , t14 authority , CrLf.
	t5 sendCommand: t15.
	t9 := t5
				getResponseUpTo: CrLf , CrLf
				ignoring: (String with: CR).
	t6 := t9 at: 1.
	t10 := t9 at: 3.
	t5 header: t6.
	t7 := t5 getHeader: 'content-length'.
	t7
		ifNotNil: [t7 := t7 asNumber].
	t12 := t5 getHeader: 'content-type'.
	t5 responseCode first = $3
		ifTrue: [t13 := t5 getHeader: 'location'.
			t13
				ifNotNil: [t13 := (t14 newFromRelativeText: t13) asString.
					self flag: #refactor.
					t5 destroy.
					^ self httpGetDocument: t13 accept: t3]].
	t11 := t5 getRestOfBuffer: t10 totalLength: t7.
	t5 responseCode = '401'
		ifTrue: [^ t6 , t11 contents].
	t5 destroy.
	^ MIMEDocument
		contentType: t12
		content: t11 contents
		url: t1! !
